/* erpcftpd: validate users against an erpcd.
 *
 * Copyright (c) David Luyer, 1999
 */

#define _BSD_SOURCE 1 /* strncasecmp() is a BSD extension */

#include <stdio.h>
#include <netdb.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>

static void die(int sig) {
  write(1, "584 timeout\r\n", 13);
  exit(-1);
}

int main(int argc, char ** argv) {
  int sd, len;
#if 0
  int id, seq;
#endif
  struct sockaddr_in Server;
  char buf1[64] = { 0x01, 0x02, 0x03, 0x04,
                    0x00, 0x20, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00,
                    0x00, 0x03, 0x00, 0x01,
		    0x00, 0x0f, /* srpc init */
		    0x00, 0x00, 0x00, 0x00, /* srpc_id */
		    0x00, 0x00, 0x00, 0x00, /* sequence # */
		    0x00 }; /* truncated key to be ignored */
  char buf2[210] = { 0x01, 0x02, 0x03, 0x04,
                     0x00, 0x20, 0x00, 0x00,
                     0x00, 0x00, 0x00, 0x00,
                     0x00, 0x03, 0x00, 0x01,
		     0x00, 0x13, /* ppp auth */
		     0x00, 0x00, 0x00, 0x00, /* srpc_id */
		     0x00, 0x00, 0x00, 0x00, /* sequence # */
		     0x00, 0x00, 0x00, 0x00, /* handle */
		     0x00, 0x00, 0x00, 0x00, /* logid */
		     0x00, 0x00, 0x00, 0x00, /* inet */
		     0x00, 0x00, /* port */
		     0x00, 0x08, /* service */
		     0x00, 0x00, 0x00, 0x00, /* direction */
		     /* + 32 user */
		     /* + 32 pw */
		     /* + 100 chars filler */ };
  char inbuf[80];

  /* BUG: udp packets aren't always accepted by the same process
   *      so sometimes they id is 0.  so use 0. */
#if 0
  srandom(getpid() ^ time(NULL));
  *(int *)(buf1+18) = id = random();
  *(int *)(buf1+22) = seq = random();
  seq = htonl((htonl(seq) + 1));
  *(int *)(buf2+18) = id;
  *(int *)(buf2+22) = seq;
#endif
  memset(&Server, 0, sizeof(Server));
  Server.sin_family = AF_INET;
  Server.sin_port = htons(121);
  write(1, "220 erpcftpd ready\r\n", 20);
  if (argc != 2) {
    write(1, "580 incorrect invocation\r\n", 26);
    return 1;
  } else {
    if ((Server.sin_addr.s_addr = inet_addr(argv[1])) == -1) {
      struct hostent *host;
      if ((host = gethostbyname(argv[1])) == NULL) {
        write(1, "581 can't find validation server\r\n", 34);
        return 1;
      }
      memcpy(&Server.sin_addr.s_addr, host->h_addr_list[0], 4);
    }
  }
  while((len = read(0, inbuf, 80))) {
    inbuf[len] = '\0';
    while(len>=0 && strchr("\r\n", inbuf[--len]))
      inbuf[len] = '\0';
    if(!strncasecmp(inbuf, "user ", 5)) {
      strncpy(buf2+46, inbuf+5, 32);
      write(1, "331 need password\r\n", 19);
    } else if(!strncasecmp(inbuf, "pass ", 5)) {
      strncpy(buf2+78, inbuf+5, 32);
      break;
    } else {
      write(1, "480 no idea what you want\r\n", 27);
    }
  }
  if (!*(buf2+46) || !*(buf2+78)) {
    write(1, "582 username or password cannot be blank\r\n", 42);
    return 1;
  }
  if ((sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    write(1, "583 socket() error\r\n", 20);
    return 1;
  }
  signal(SIGALRM, die); /* 584 */
  alarm(60);
  if (sendto(sd, buf1, 27, 0, &Server, sizeof(Server)) < 0) {
    write(1, "585 sendto() error\r\n", 20);
    return 1;
  }
  if (recv(sd, buf1, sizeof(buf1), 0) < 0) {
    write(1, "586 recv() error\r\n", 18);
    return 1;
  }
  if(*(buf1+4+2+1) != 2) {
    write(1, "587 server error 1\r\n", 20);
    return 1;
  }
  if (sendto(sd, buf2, 210, 0, &Server, sizeof(Server)) < 0) {
    write(1, "588 sendto/2() error\r\n", 22);
    return 1;
  }
  if (recv(sd, buf2, sizeof(buf2), 0) != 26) {
    write(1, "589 recv/2() error\r\n", 20);
    return 1;
  }
  if(*(buf2+4+2+1) != 2 || *(int *)(buf2+22) != htonl(0xbabe3333ul)) {
    write(1, "590 server error 2\r\n", 20);
    return 1;
  }
  if (recv(sd, buf2, sizeof(buf2), 0) != 34) {
    write(1, "591 recv/3() error\r\n", 20);
    return 1;
  }
  if(*(buf2+4+2+1) != 0 || (*(int *)(buf2+30) != htonl(0xface1111ul) &&
                            *(int *)(buf2+30) != htonl(0xdeaf2222ul))) {
    write(1, "592 server error 3\r\n", 20);
    return 1;
  }
  if(*(int *)(buf2+30) != htonl(0xface1111ul)) {
    write(1, "530 password incorrect\r\n", 24);
    return 1;
  }
  close(sd);
  write(1, "230 welcome\r\n", 13);
  while((len=read(0, inbuf, 80))) {
    inbuf[len] = '\0';
    while(len>=0 && strchr("\r\n", inbuf[--len]))
      inbuf[len] = '\0';
    if(!strncasecmp(inbuf, "quit", 4)) {
      write(1, "221 bye\r\n", 9);
      return 0;
    } else {
      write(1, "480 no idea what you want\r\n", 27);
    }
  }
  write(1, "221 bye\r\n", 9);
  return 0;
}
